/****************************************************************************
 * $Revision: 1.2 $  $Author: reids $  $Date: 2001/11/14 17:58:54 $
 *
 * PROJECT:      Distributed Robotic Agents
 * DESCRIPTION:  
 *
 * (c) Copyright 1999 CMU. All rights reserved.
 ***************************************************************************/

/***************************************************************************
 * INCLUDES
 ***************************************************************************/

#include <assert.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include <fstream>

#include "XDRParser.h"
#include "XDROutputSpec.h"
#include "XDRMapNames.h"
#include "XDRFormatIPC.h"
#include "XDROutputHeader.h"
#include "xdrgen.h"

/***************************************************************************
 * GLOBAL VARIABLES AND STATIC MEMBER FIELDS
 ***************************************************************************/

string defaultLangG = "c++";
string xdrgenVersionG = "$Revision: 1.2 $ $Date: 2001/11/14 17:58:54 $";

/***************************************************************************
 * FUNCTIONS
 ***************************************************************************/

void
usage(void) {
  cerr << "usage: xdrgen OPTIONS <xdrFile> [outputHeaderFile]" << endl
       << " -h or --help        Print this help" << endl
       << " --lang=[c,c++]      Change language for header output" << endl
       << "                     (default: " << defaultLangG << ")" << endl
       << " -D<name>=<value>, -I<directory>" << endl
       << "                     These options are passed on to the C" << endl
       << "                     pre-processor before xdrgen parses the" << endl
       << "                     input file (see cpp(1) or 'info cpp')" << endl
       << "### DEBUGGING OPTIONS" << endl
       << " -v or --verbose     Talk about parse steps" << endl
       << " -n or --no-header   Don't output the header" << endl
       << " --spec              Print out parse tree as XDR spec" << endl
       << " --formats           Print out IPC formats for all types" << endl;
  exit(EXIT_FAILURE);
}

static HeaderLangEnum
langFromString(string langString) {
  if (langString == "c++") {
    return LANG_CPLUSPLUS;
  } else if (langString == "c") {
    return LANG_C;
  } else {
    cerr << "xdrgen: unknown language " << langString << " specified"
	 << endl;
    usage();
    return LANG_C; // will never reach here
  }
}

int
main(int argc, char *argv[]) {
  int argi;
  string args;
  bool pastOptions = false;
  bool noHeader = false;
  bool debugPrintSpec = false;
  bool debugPrintFormats = false;
  bool verbose = false;
  HeaderLangEnum lang = langFromString(defaultLangG.c_str());
  char *inputFileName = 0;
  char *outputFileName = 0;
  list<string> cppArgs;

  for (argi=1; argi < argc; argi++) {
    args = argv[argi];

    if (args == "--") {
      pastOptions = true;
    }
    if (!pastOptions && args.find("-") == 0) {
      if (args == "-h" || args == "--help") {
	usage();
      } else if (args.find("--lang=") == 0) {
	string langString = args.substr(string("--lang=").size());
	lang = langFromString(langString);
      } else if (args == "-v" || args == "--verbose") {
	verbose = true;
      } else if (args == "-n" || args == "--no-header") {
	noHeader = true;
      } else if (args == "--spec") {
	debugPrintSpec = true;
      } else if (args == "--formats") {
	debugPrintFormats = true;
      } else if (   args.find("-D") == 0
		 || args.find("-I") == 0) {
	cppArgs.push_back(args);
      } else {
	cerr << "xdrgen: unknown option " << args << endl;
	usage();
      }
    } else {
      if (0 != inputFileName) {
	if (0 != outputFileName) {
	  cerr << "xdrgen: too many arguments" << endl;
	  usage();
	} else {
	  outputFileName = argv[argi];
	}
      } else {
	inputFileName = argv[argi];
      }
    }
  }
  if (0 == inputFileName) {
    cerr << "xdrgen: not enough arguments" << endl;
    usage();
  }

  XDRParser *parser = new XDRParser();

  if (verbose) cout << endl << "--- parsing" << endl;
  if (!parser->parseFile(inputFileName,cppArgs)) {
    fprintf(stderr, "couldn't parse file\n");
    exit(EXIT_FAILURE);
  }

  XDRSpecification *spec = parser->getSyntaxTree();
  if (debugPrintSpec) {
    if (verbose) cout << endl << "--- printing parse tree as XDR spec" << endl;
    XDROutputSpec outSpec(cout);
    outSpec.iterate(spec);
  }

  // name mapping is required before generating IPC formats.
  // it modifies the parse tree so that type name identifiers
  // resolve to the right types.
  if (verbose) cout << endl << "--- mapping identifier names" << endl;
  XDRMapNames *nameMapper = new XDRMapNames;
  nameMapper->iterate(spec);

  if (debugPrintFormats) {
    if (verbose) cout << endl << "--- outputting IPC formats" << endl;
    DebugFormatIPC debugFmt(cout);
    debugFmt.iterate(spec);
  }
  
  if (!noHeader) {
    if (verbose) cout << endl << "--- outputting C header" << endl;
    ostream *outStreamP;
    if (0 == outputFileName || outputFileName == "-") {
      outStreamP = &cout;
    } else {
      outStreamP = new ofstream(outputFileName);
      if (! (*outStreamP)) {
	cerr << "xdrgen: couldn't open `" << outputFileName
	     << "' for writing" << endl;
	exit(EXIT_FAILURE);
      }
    }
    XDROutputHeader outHeader(*outStreamP,outputFileName,lang);
    outHeader.iterate(spec);
  }

  return EXIT_SUCCESS;
}

/***************************************************************************
 * REVISION HISTORY:
 * $Log: xdrgen.cc,v $
 * Revision 1.2  2001/11/14 17:58:54  reids
 * Incorporating various changes that Trey made in the DIRA/FIRE versions.
 *
 * Revision 1.1  2001/03/16 17:56:07  reids
 * Release of Trey's code to generate IPC format strings from XDR definitions.
 *
 * Revision 1.2  2001/08/27 19:20:30  trey
 * fixed unnecessary dependency on DIRA logging.h
 *
 * Revision 1.1  2001/08/27 18:44:03  trey
 * initial check-in
 *
 * Revision 1.5  2001/03/21 19:26:55  trey
 * added ability of xdrgen to pass on -D and -I flags to xdrcpp
 *
 * Revision 1.4  2001/02/13 16:33:53  trey
 * switched include of headers under src to use "dir/foo.h" instead of <dir/foo.h>; dependency checking is now easier. made some corresponding makefile changes
 *
 * Revision 1.3  2001/02/08 00:41:59  trey
 * added external IPC format feature to xdrgen; we also now tag the generated file with a version string
 *
 * Revision 1.2  2001/02/06 02:01:55  trey
 * fixed bugs discovered while generating commonTypes message formats
 *
 * Revision 1.1  2001/02/05 21:10:52  trey
 * initial check-in
 *
 * Revision 1.3  2000/02/25 22:14:12  trey
 * added sanity check of parseFile() results
 *
 * Revision 1.2  1999/11/11 15:14:59  trey
 * changed to support logging
 *
 * Revision 1.1  1999/11/08 15:44:50  trey
 * renamed RCLmain to rcl and made it rely on RCLParser
 *
 * Revision 1.1  1999/11/03 19:31:37  trey
 * initial check-in
 *
 * Revision 1.1  1999/11/01 17:30:14  trey
 * initial check-in
 *
 *
 ***************************************************************************/
